/*
 * Copyright 2013 JBoss Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kie.workbench.common.screens.projecteditor.client.forms;

import java.util.List;
import javax.inject.Inject;

import com.github.gwtbootstrap.client.ui.Button;
import com.github.gwtbootstrap.client.ui.ButtonCell;
import com.github.gwtbootstrap.client.ui.DataGrid;
import com.google.gwt.cell.client.EditTextCell;
import com.google.gwt.cell.client.FieldUpdater;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import org.guvnor.common.services.project.model.ListenerModel;
import org.guvnor.common.services.project.model.WorkItemHandlerModel;
import org.kie.workbench.common.screens.projecteditor.client.resources.i18n.ProjectEditorConstants;

public class WorkItemHandlersPanelViewImpl
        extends Composite
        implements WorkItemHandlersPanelView {

    private Presenter presenter;

    interface Binder
            extends
            UiBinder<Widget, WorkItemHandlersPanelViewImpl> {

    }

    private static Binder uiBinder = GWT.create(Binder.class);

    @UiField(provided = true)
    DataGrid<WorkItemHandlerModel> grid;

    @UiField
    Button addButton;

    @Inject
    public WorkItemHandlersPanelViewImpl() {
        grid = new DataGrid<WorkItemHandlerModel>();

        grid.setEmptyTableWidget(new Label("---"));
        grid.setBordered(true);

        addNameColumn();
        addTypeColumn();
        addDeleteColumn();

        initWidget(uiBinder.createAndBindUi(this));
    }

    private void addDeleteColumn() {
        Column<WorkItemHandlerModel, String> column = new Column<WorkItemHandlerModel, String>(new ButtonCell()) {
            @Override
            public String getValue(WorkItemHandlerModel object) {
                return ProjectEditorConstants.INSTANCE.Delete();
            }
        };

        column.setFieldUpdater(new FieldUpdater<WorkItemHandlerModel, String>() {
            @Override
            public void update(int index, WorkItemHandlerModel model, String value) {
                presenter.onDelete(model);
            }
        });

        grid.addColumn(column);
        grid.setColumnWidth(column, "70px");
    }

    public void setPresenter(Presenter presenter) {
        this.presenter = presenter;
    }

    private void addTypeColumn() {

        Column<WorkItemHandlerModel, String> column = new Column<WorkItemHandlerModel, String>(new EditTextCell()) {
            @Override
            public String getValue(WorkItemHandlerModel model) {
                return model.getType();
            }
        };

        column.setFieldUpdater(new FieldUpdater<WorkItemHandlerModel, String>() {
            @Override
            public void update(int index, WorkItemHandlerModel model, String value) {
                model.setType(value);
            }
        });

        grid.addColumn(column, ProjectEditorConstants.INSTANCE.Type());
    }

    private void addNameColumn() {

        Column<WorkItemHandlerModel, String> column = new Column<WorkItemHandlerModel, String>(new EditTextCell()) {
            @Override
            public String getValue(WorkItemHandlerModel model) {
                return model.getName();
            }
        };

        column.setFieldUpdater(new FieldUpdater<WorkItemHandlerModel, String>() {
            @Override
            public void update(int index, WorkItemHandlerModel model, String value) {
                model.setName(value);
            }
        });

        grid.addColumn(column, ProjectEditorConstants.INSTANCE.Name());
    }

    public void setModels(List<WorkItemHandlerModel> handlerModels) {
        grid.setRowData(handlerModels);
    }

    @UiHandler("addButton")
    public void onAddClick(ClickEvent event) {
        presenter.onAdd();
    }

}
